import { Component, inject, Inject, Input } from '@angular/core';
import { FormControl, FormGroup, ReactiveFormsModule, Validators } from "@angular/forms";
import { Hotel } from "../HotelItem/hotel";
import { HttpClient } from '@angular/common/http';
import { catchError } from "rxjs";
import { Router, RouterLink } from "@angular/router";
import { NgIf } from "@angular/common";
import { MatButtonModule } from "@angular/material/button";
import { MatIconModule } from "@angular/material/icon";

@Component({
  selector: 'app-hotel-form',
  standalone: true,
  imports: [
    ReactiveFormsModule,
    RouterLink,
    NgIf,
    MatButtonModule,
    MatIconModule
  ],
  templateUrl: './hotel-form.component.html',
  styleUrl: './hotel-form.component.css'
})
export class HotelFormComponent {
  public form!: FormGroup;

  @Input()
  public hotel!: Hotel;

  public router: Router = inject(Router);

  constructor(private http: HttpClient) {}

  ngOnInit() {
    this.form = new FormGroup({
      name: new FormControl(this.hotel?.hotelName || '', [Validators.minLength(3), Validators.required]),
      description: new FormControl(this.hotel?.description || '', [Validators.minLength(3), Validators.required]),
      price: new FormControl(this.hotel?.price || '', [Validators.min(0), Validators.required]),
      rating: new FormControl(this.hotel?.rating || '', [Validators.min(0), Validators.max(5), Validators.required])
    });
  }

  public delete(id: number) {
    this.http.delete(`api/hotels/${this.hotel?.id}`).pipe(
      catchError(err => {
        console.error('Error deleting hotel', err);
        return err;
      })
    ).subscribe(() => {
      console.log('Hotel deleted');
    });

    this.router.navigate(['/']);
  }

  public submit() {
    if (this.form.valid) {
      const hotelData: Hotel = {
        id: this.hotel?.id || 0,
        hotelName: this.form.value.name,
        description: this.form.value.description,
        price: this.form.value.price,
        rating: this.form.value.rating,
        imageUrl: this.hotel?.imageUrl || 'https://placehold.co/2000x1050/EEE/31343C',
        tags: this.hotel?.tags || []
      };

      if (this.hotel) {
        this.http.put(`api/hotels/${this.hotel.id}`, hotelData).pipe(
          catchError(err => {
            console.error('Error updating hotel', err);
            return err;
          })
        ).subscribe(() => {
          console.log('Hotel updated');
        });
      }

      else {
        this.http.post('api/hotels', hotelData).pipe(
          catchError(err => {
            console.error('Error creating hotel', err);
            return err;
          })
        ).subscribe(() => {
          console.log('Hotel created');
        });
      }

      this.router.navigate(['/']);
    }
  }
}
